import _toConsumableArray from '@babel/runtime/helpers/esm/toConsumableArray';
import _slicedToArray from '@babel/runtime/helpers/esm/slicedToArray';
import _defineProperty from '@babel/runtime/helpers/esm/defineProperty';
import { createStore, combineReducers, applyMiddleware, compose } from 'redux';
import { connect } from 'react-redux';
export { Provider, batch } from 'react-redux';

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
/**
 * Utils
 */

var ERR = {
  NOT_STRING: function NOT_STRING(name) {
    return "'".concat(name, "' should be a string");
  },
  NOT_ARRAY: function NOT_ARRAY(name) {
    return "'".concat(name, "' should be an array");
  },
  NOT_OBJECT: function NOT_OBJECT(name) {
    return "'".concat(name, "' should be an object");
  },
  NOT_CLASS: function NOT_CLASS(name) {
    return "'".concat(name, "' should be a class");
  },
  NO_DISPATCH: function NO_DISPATCH() {
    return "Please don't use 'dispatch' directly in Retalk";
  },
  NOT_EXIST: function NOT_EXIST(name) {
    return "'".concat(name, "' Model dose not exist");
  }
};

var isObj = function isObj(obj) {
  return Object.prototype.toString.call(obj) === '[object Object]';
};
/**
 * Add a model, create the model's reducer
 *
 * @param {string} name
 * @param {class} Model
 * @param {object} newModels
 * @param {object} newReducers
 */


var createReducer = function createReducer(name, Model, newModels, newReducers) {
  var model = _objectSpread({}, new Model());

  newModels[name] = model;
  model.models = newModels;

  newReducers[name] = function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : model.state || {};
    var action = arguments.length > 1 ? arguments[1] : undefined;

    var _action$type$split = action.type.split('/'),
        _action$type$split2 = _slicedToArray(_action$type$split, 1),
        modelName = _action$type$split2[0];

    if (modelName === name) return action.newState;
    return state;
  };
};
/**
 * Format a model's actions
 *
 * @param {string} name
 * @param {class} Model
 * @param {object} newModels
 * @param {function} newDispatch
 * @param {function} realDispatch
 */


var createActions = function createActions(name, Model, newModels, newDispatch, realDispatch) {
  var model__proto__ = Model.prototype;
  var model = newModels[name];

  var setLoading = function setLoading(actionName, type, loading) {
    model[actionName].loading = loading;
    return realDispatch({
      type: "".concat(name, "/").concat(actionName, "/").concat(type, "_LOADING"),
      newState: _objectSpread(_objectSpread({}, model.state), {}, {
        loading: _defineProperty({}, actionName, loading)
      })
    });
  };

  newDispatch[name] = {};
  Object.getOwnPropertyNames(model__proto__).forEach(function (actionName) {
    if (actionName === 'constructor' || actionName === 'models') return;

    var setState = function setState(payload) {
      var newState = Object.assign(model.state, payload);
      return realDispatch({
        type: "".concat(name, "/").concat(actionName),
        newState: _objectSpread({}, newState)
      });
    };

    var oldAction;

    var newAction = function newAction() {
      if (!oldAction) oldAction = model__proto__[actionName].bind(_objectSpread(_objectSpread({}, model), {}, {
        setState: setState
      }));
      var res = oldAction.apply(void 0, arguments);
      if (!res || typeof res.then !== 'function') return res;
      setLoading(actionName, 'START', true);
      return res.finally(function () {
        setLoading(actionName, 'STOP', false);
      });
    };

    model[actionName] = newAction;
    newDispatch[name][actionName] = newAction;
  });
};
/**
 * Initialize all models
 *
 * @param {object} models
 * @param {array} middleware
 * @return {object} Store
 */


var setStore = function setStore() {
  var models = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var middleware = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var __DEV__ = process.env.NODE_ENV !== 'production';

  if (__DEV__) {
    if (!isObj(models)) throw new Error(ERR.NOT_OBJECT('models'));
    if (!Array.isArray(middleware)) throw new Error(ERR.NOT_ARRAY('middleware'));
  }

  var newModels = {};
  var newReducers = {};
  var nameModelList = Object.entries(models); // Reducers

  if (__DEV__) {
    nameModelList.forEach(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          name = _ref2[0],
          Model = _ref2[1];

      if (typeof Model !== 'function') throw new Error(ERR.NOT_CLASS('Model'));
      createReducer(name, Model, newModels, newReducers);
    });
  } else {
    nameModelList.forEach(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          name = _ref4[0],
          Model = _ref4[1];

      createReducer(name, Model, newModels, newReducers);
    });
  } // Store


  var __COMPOSE__ = window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__;
  var newCompose = __DEV__ && __COMPOSE__ ? __COMPOSE__ : compose;
  var store = createStore(combineReducers(newReducers), newCompose(applyMiddleware.apply(void 0, _toConsumableArray(middleware))));
  var realDispatch = store.dispatch;

  var newDispatch = function newDispatch() {
    console.error(ERR.NO_DISPATCH());
  };

  store.dispatch = newDispatch; // Actions

  nameModelList.forEach(function (_ref5) {
    var _ref6 = _slicedToArray(_ref5, 2),
        name = _ref6[0],
        Model = _ref6[1];

    createActions(name, Model, newModels, newDispatch, realDispatch);
  }); // store.add

  store.add = function (modelsToAdd) {
    if (__DEV__) {
      if (!isObj(modelsToAdd)) throw new Error(ERR.NOT_OBJECT('models'));
      Object.entries(modelsToAdd).forEach(function (_ref7) {
        var _ref8 = _slicedToArray(_ref7, 2),
            name = _ref8[0],
            Model = _ref8[1];

        if (name in newModels) return;
        if (typeof Model !== 'function') throw new Error(ERR.NOT_CLASS('Model'));
        createReducer(name, Model, newModels, newReducers);
        store.replaceReducer(combineReducers(newReducers));
        createActions(name, Model, newModels, newDispatch, realDispatch);
      });
    } else {
      Object.entries(modelsToAdd).forEach(function (_ref9) {
        var _ref10 = _slicedToArray(_ref9, 2),
            name = _ref10[0],
            Model = _ref10[1];

        if (name in newModels) return;
        createReducer(name, Model, newModels, newReducers);
        store.replaceReducer(combineReducers(newReducers));
        createActions(name, Model, newModels, newDispatch, realDispatch);
      });
    }
  };

  return store;
};
/**
 * Eject some models to a component
 *
 * @param {string|object|function} names
 * @return {function} Higher-order component
 */

var withStore = function withStore() {
  for (var _len = arguments.length, names = new Array(_len), _key = 0; _key < _len; _key++) {
    names[_key] = arguments[_key];
  }

  var __DEV__ = process.env.NODE_ENV !== 'production';

  var first = names[0]; // ['a', 'b']

  if (typeof first === 'string') {
    var mapStore = function mapStore(store) {
      var obj = {};
      names.forEach(function (name) {
        obj = _objectSpread(_objectSpread({}, obj), store[name]);
      });
      return obj;
    };

    var mapState;

    if (__DEV__) {
      mapState = function mapState(state) {
        var stateObj = {};
        names.forEach(function (name, index) {
          if (index > 0 && typeof name !== 'string') throw new Error(ERR.NOT_STRING('name'));
          var modelState = state[name];
          if (typeof modelState === 'undefined') throw new Error(ERR.NOT_EXIST(name));
          stateObj = _objectSpread(_objectSpread({}, stateObj), modelState);
        });
        return stateObj;
      };
    } else {
      mapState = mapStore;
    }

    return connect(mapState, mapStore);
  } // [{ a: ['a1', 'a2'] }]


  if (isObj(first)) {
    var stateActionsMap = first;
    var actionsMap = {};

    var _mapState;

    if (__DEV__) {
      _mapState = function _mapState(state) {
        var stateObj = {};
        Object.entries(stateActionsMap).forEach(function (_ref11) {
          var _ref12 = _slicedToArray(_ref11, 2),
              name = _ref12[0],
              keys = _ref12[1];

          actionsMap[name] = [];
          var modelState = state[name];
          if (typeof modelState === 'undefined') throw new Error(ERR.NOT_EXIST(name));
          keys.forEach(function (key) {
            var val = modelState[key];
            typeof val !== 'undefined' ? stateObj[key] = val : actionsMap[name].push(key);
          });
          stateObj.loading = _objectSpread(_objectSpread({}, stateObj.loading), modelState.loading);
        });
        return stateObj;
      };
    } else {
      _mapState = function _mapState(state) {
        var stateObj = {};
        Object.entries(stateActionsMap).forEach(function (_ref13) {
          var _ref14 = _slicedToArray(_ref13, 2),
              name = _ref14[0],
              keys = _ref14[1];

          actionsMap[name] = [];
          var modelState = state[name];
          keys.forEach(function (key) {
            var val = modelState[key];
            typeof val !== 'undefined' ? stateObj[key] = val : actionsMap[name].push(key);
          });
          stateObj.loading = _objectSpread(_objectSpread({}, stateObj.loading), modelState.loading);
        });
        return stateObj;
      };
    }

    var mapDispatch = function mapDispatch(dispatch) {
      var actionsObj = {};
      Object.entries(actionsMap).forEach(function (_ref15) {
        var _ref16 = _slicedToArray(_ref15, 2),
            name = _ref16[0],
            keys = _ref16[1];

        var modelActions = dispatch[name];
        keys.forEach(function (key) {
          actionsObj[key] = modelActions[key];
        });
      });
      return actionsObj;
    };

    return connect(_mapState, mapDispatch);
  } // connect


  return connect.apply(void 0, names);
};

export { setStore, withStore };
